@@ -65,10 +65,12 @@ typedef struct _sen_io_header {
   uint32_t header_size;
   uint32_t segment_size;
   uint32_t max_segment;
+  uint32_t reserved;
   uint64_t curr_size;
-  uint32_t reserved[7];
 } io_header;
 
+#define IO_HEADER_SIZE 64
+
 inline static unsigned int sen_get_pagesize(void);
 inline static sen_rc sen_open(fileinfo *fi, const char *path, int flags, size_t maxsize);
 inline static void sen_fileinfo_init(fileinfo *fis, int nfis);
@@ -91,7 +93,7 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
   io_header *header;
   if (!path || !*path || (strlen(path) > PATH_MAX - 4)) { return NULL; }
   if (!pagesize) { pagesize = sen_get_pagesize(); }
-  total_header_size = sizeof(io_header) + max_segment * sizeof(uint32_t) + header_size;
+  total_header_size = IO_HEADER_SIZE + max_segment * sizeof(uint32_t) + header_size;
   for (b = pagesize; b < total_header_size; b += pagesize);
   bs = (b + segment_size - 1) / segment_size;
   max_nfiles = (unsigned int)(
@@ -111,7 +113,7 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
           memset(io->maps, 0, sizeof(sen_io_mapinfo) * max_segment);
           strncpy(io->path, path, PATH_MAX);
           io->header = header;
-          io->nrefs = (uint32_t *)(((byte *) header) + sizeof(io_header));
+          io->nrefs = (uint32_t *)(((byte *) header) + IO_HEADER_SIZE);
           io->user_header = ((byte *) io->nrefs) + max_segment * sizeof(uint32_t);
           io->base = b;
           io->base_seg = bs;
@@ -162,9 +164,9 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
     if (!segment_size) { return NULL; }
   }
   if (v08p) {
-    total_header_size = sizeof(io_header) + header_size;
+    total_header_size = IO_HEADER_SIZE + header_size;
   } else {
-    total_header_size = sizeof(io_header) + header_size + max_segment * sizeof(uint32_t);
+    total_header_size = IO_HEADER_SIZE + header_size + max_segment * sizeof(uint32_t);
   }
   for (b = pagesize; b < total_header_size; b += pagesize);
   bs = (b + segment_size - 1) / segment_size;
@@ -184,7 +186,7 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
             io->nrefs = SEN_CALLOC(sizeof(uint32_t) * max_segment);
             io->user_header = ((byte *) header) + sizeof(io_header08);
           } else {
-            io->nrefs = (uint32_t *)(((byte *) header) + sizeof(io_header));
+            io->nrefs = (uint32_t *)(((byte *) header) + IO_HEADER_SIZE);
             io->user_header = ((byte *) io->nrefs) + max_segment * sizeof(uint32_t);
           }
           if (io->nrefs) {
