@@ -15,7 +15,6 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include "senna_in.h"
-#include <stdio.h>              /* for debug */
 #include <string.h>
 #include <stddef.h>
 #include <assert.h>
@@ -33,8 +32,8 @@ int
 sen_bm_check_euc(const unsigned char *x, const size_t y)
 {
   const unsigned char *p;
-  for (p = x + y - 1; p >= x && *p >= 0x80; p--);
-  return (int) ((x + y - p) % 2);
+  for (p = x + y - 1; p >= x && *p >= 0x80U; p--);
+  return (int) ((x + y - p) & 1);
 }
 
 int
@@ -42,9 +41,9 @@ sen_bm_check_sjis(const unsigned char *x, const size_t y)
 {
   const unsigned char *p;
   for (p = x + y - 1; p >= x; p--)
-    if ((*p < 0x81) || (*p > 0x9F && *p < 0xE0) || (*p > 0xFC))
+    if ((*p < 0x81U) || (*p > 0x9fU && *p < 0xe0U) || (*p > 0xfcU))
       break;
-  return (int) ((x + y - p) % 2);
+  return (int) ((x + y - p) & 1);
 }
 
 void
@@ -62,7 +61,7 @@ sen_bm_suffixes(const unsigned char *x, size_t m, size_t *suff)
       if (i < (intptr_t) g)
         g = i;
       f = i;
-      while (g >= 0 && x[g] == x[g + m - 1 - f])
+      while (g > 0 && x[g] == x[g + m - 1 - f])
         --g;
       suff[i] = f - g;
     }
@@ -122,7 +121,7 @@ sen_bm_preBmBc(const unsigned char *x, size_t m, size_t *bmBc)
 }
 
 inline void
-sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n, sen_encoding encoding)
+sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n)
 {
   register unsigned char *limit, ck;
   register const unsigned char *p, *cp;
@@ -132,7 +131,7 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n, sen_encoding encodi
   unsigned char *y;
   size_t shift, found;
 
-  const size_t m = cond->keyword_len;
+  const size_t m = cond->keyword->norm_blen;
 
   y = (unsigned char *) object->norm;
   if (m == 1) {
@@ -161,32 +160,25 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n, sen_encoding encodi
   if (n - cond->found > 12 * m) {
     limit = y + n - 11 * m;
     while (p <= limit) {
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      if (delta1 == 0)
+      p += bmBc[p[-1]];
+      if(!(delta1 = bmBc[p[-1]])) {
         goto check;
-      delta1 = bmBc[p[-1]];
+      }
       p += delta1;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      if (delta1 == 0)
+      p += bmBc[p[-1]];
+      p += bmBc[p[-1]];
+      if(!(delta1 = bmBc[p[-1]])) {
         goto check;
-      delta1 = bmBc[p[-1]];
+      }
       p += delta1;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      if (delta1 == 0)
+      p += bmBc[p[-1]];
+      p += bmBc[p[-1]];
+      if(!(delta1 = bmBc[p[-1]])) {
         goto check;
-      delta1 = bmBc[p[-1]];
-      p += delta1;
-      delta1 = bmBc[p[-1]];
+      }
       p += delta1;
+      p += bmBc[p[-1]];
+      p += bmBc[p[-1]];
       continue;
     check:
       SEN_BM_BM_COMPARE;
@@ -196,13 +188,13 @@ sen_bm_TunedBM(_snip_cond *cond, sen_nstr *object, size_t n, sen_encoding encodi
   /* limit check + search */
   limit = y + n;
   delta1 = bmBc[p[-1]];
-  while (p <= limit) {
-    delta1 = bmBc[(p += delta1)[-1]];
-    if (delta1 != 0)
-      continue;
-    SEN_BM_BM_COMPARE;
-    delta1 = shift;
-  }
+  while(p <= limit) {
+    p += delta1;
+    if (!(delta1 = bmBc[p[-1]])) {
+      SEN_BM_BM_COMPARE;
+      p += shift;
+    }
+  };
   cond->stopflag = 1;
 
 }
@@ -253,6 +245,7 @@ sen_snip_add_cond(sen_snip *snip,
                   const char *closetag, unsigned int closetag_len)
 {
   _snip_cond *cond = snip->cond + snip->cond_len;
+  size_t norm_key_blen;
 
   if (!keyword || !*keyword || snip->cond_len >= MAX_SNIP_COND_COUNT) {
     return sen_invalid_argument;
@@ -274,31 +267,29 @@ sen_snip_add_cond(sen_snip *snip,
   }
 
   if (snip->flags & SEN_SNIP_NORMALIZE) {
-    cond->keyword = sen_nstr_open(keyword, snip->encoding, SEN_STR_REMOVEBLANK);        /* normalize dup */
+    cond->keyword = sen_nstr_open(keyword, keyword_len,
+                                  snip->encoding, SEN_STR_REMOVEBLANK);
     if (!cond->keyword) {
       SEN_LOG(sen_log_alert, "sen_nstr_open on sen_snip_add_cond failed !");
       return sen_memory_exhausted;
     }
   } else {
-    sen_nstr *nstr;
-    if (!(nstr = SEN_MALLOC(sizeof(sen_nstr)))) {
-      SEN_LOG(sen_log_alert, "sen_nstr_open on sen_snip_add_cond failed !");
+    cond->keyword = sen_fakenstr_open(keyword, keyword_len, snip->encoding, SEN_STR_REMOVEBLANK);
+    if (!cond->keyword) {
+      SEN_LOG(sen_log_alert, "sen_fakenstr_open on sen_snip_add_cond failed !");
       return sen_memory_exhausted;
     }
-    nstr->norm = SEN_STRDUP(keyword);   /* dup, the label is 'norm' but not normalized  */
-    nstr->ctypes = NULL;
-    nstr->checks = NULL;
-    cond->keyword = nstr;
   }
-  cond->keyword_len = strlen(cond->keyword->norm); /* byte length, not cond->keyword->length */
-  if (!cond->keyword_len || cond->keyword_len > snip->width) {
+  norm_key_blen = cond->keyword->norm_blen; /* byte length, not cond->keyword->length */
+  if (!norm_key_blen || norm_key_blen > snip->width) {
     snip_cond_close(cond);
     return sen_invalid_argument;
   }
 
-  sen_bm_preBmBc((unsigned char *) cond->keyword->norm, cond->keyword_len, cond->bmBc);
-  cond->shift = cond->bmBc[(unsigned char) cond->keyword->norm[cond->keyword_len - 1]];
-  cond->bmBc[(unsigned char) cond->keyword->norm[cond->keyword_len - 1]] = 0;
+  sen_bm_preBmBc((unsigned char *) cond->keyword->norm, norm_key_blen, cond->bmBc);
+  cond->shift = cond->bmBc[
+                (unsigned char) cond->keyword->norm[norm_key_blen - 1]];
+  cond->bmBc[(unsigned char) cond->keyword->norm[norm_key_blen - 1]] = 0;
 
   snip->cond_len++;
   return sen_success;
@@ -318,7 +309,7 @@ sen_snip_find_firstbyte(const char *string, sen_encoding encoding, size_t offset
       offset += doffset;
     break;
   case sen_enc_utf8:
-    while (string[offset] <= (char) 0xc0)
+    while (string[offset] <= (char)0xc0)
       offset += doffset;
     break;
   default:
@@ -381,87 +372,20 @@ exec_clean(sen_snip *snip)
 sen_rc
 sen_snip_close(sen_snip *snip)
 {
-  size_t i;
+  _snip_cond *cond, *cond_end;
   if (!snip) {
     return sen_invalid_argument;
   }
-  for (i = 0; i < snip->cond_len; i++) {
-    snip_cond_close(snip->cond + i);
+  cond = cond_end = snip->cond;
+  cond_end += snip->cond_len;
+  for (; cond < cond_end; cond++) {
+    snip_cond_close(cond);
   }
   exec_clean(snip);
   SEN_FREE(snip);
   return sen_success;
 }
 
-sen_nstr *
-sen_fakenstr_open(const char *string, size_t string_blen, sen_encoding encoding,
-                  int flags)
-{
-  sen_nstr *nstr;
-  int_least8_t f;
-  unsigned char c;
-  size_t i;
-
-  if (!(nstr = SEN_MALLOC(sizeof(sen_nstr)))) {
-    return NULL;
-  }
-  nstr->norm = SEN_STRDUP(string);
-  nstr->ctypes = NULL;
-  if (!(nstr->checks = (int_least8_t *) SEN_MALLOC(sizeof(int_least8_t) * string_blen))) {
-    SEN_FREE(nstr);
-    return NULL;
-  }
-
-  f = 0;
-  switch (encoding) {
-  case sen_enc_euc_jp:
-    for (i = 0; i < string_blen; i++) {
-      if (!f) {
-        c = (unsigned char) string[i];
-        f = ((c >= 0xa1 && c <= 0xfe) || c == 0x8e ? 2 : (c == 0x8f ? 3 : 1)
-          );
-        nstr->checks[i] = f;
-      } else {
-        nstr->checks[i] = 0;
-      }
-      f--;
-    }
-    break;
-  case sen_enc_sjis:
-    for (i = 0; i < string_blen; i++) {
-      if (!f) {
-        c = (unsigned char) string[i];
-        f = (c >= 0x81 && ((c <= 0x9f) || (c >= 0xe0 && c <= 0xfc)) ? 2 : 1);
-        nstr->checks[i] = f;
-      } else {
-        nstr->checks[i] = 0;
-      }
-      f--;
-    }
-    break;
-  case sen_enc_utf8:
-    for (i = 0; i < string_blen; i++) {
-      if (!f) {
-        c = (unsigned char) string[i];
-        f = (c & 0x80 ? (c & 0x20 ? (c & 0x10 ? 4 : 3)
-                         : 2)
-             : 1);
-        nstr->checks[i] = f;
-      } else {
-        nstr->checks[i] = 0;
-      }
-      f--;
-    }
-    break;
-  default:
-    for (i = 0; i < string_blen; i++) {
-      nstr->checks[i] = 1;
-    }
-    break;
-  }
-  return nstr;
-}
-
 sen_rc
 sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
               unsigned int *nresults, unsigned int *max_tagged_len)
@@ -476,7 +400,8 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
   }
   if (snip->flags & SEN_SNIP_NORMALIZE) {
     snip->nstr =
-      sen_nstr_open(string, snip->encoding, SEN_STR_WITH_CHECKS | SEN_STR_REMOVEBLANK);
+      sen_nstr_open(string, string_len, snip->encoding,
+                    SEN_STR_WITH_CHECKS | SEN_STR_REMOVEBLANK);
   } else {
     snip->nstr =
       sen_fakenstr_open(string, string_len, snip->encoding,
@@ -487,10 +412,10 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
     SEN_LOG(sen_log_alert, "sen_nstr_open on sen_snip_exec failed !");
     return sen_memory_exhausted;
   }
-  nstr_blen = strlen(snip->nstr->norm); /* byte length, not nstr->length */
+  nstr_blen = snip->nstr->norm_blen; /* byte length, not nstr->length */
 
   for (i = 0; i < snip->cond_len; i++) {
-    sen_bm_TunedBM(snip->cond + i, snip->nstr, nstr_blen, snip->encoding);
+    sen_bm_TunedBM(snip->cond + i, snip->nstr, nstr_blen);
   }
 
   {
@@ -506,7 +431,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
       snip_result->tag_count = 0;
 
       while (1) {
-        size_t min_start_offset = (size_t) - 1;
+        size_t min_start_offset = (size_t) -1;
         size_t max_end_offset = 0;
         _snip_cond *cond = NULL;
 
@@ -536,7 +461,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
               }
             }
             if (exclude_other_cond) {
-              sen_bm_TunedBM(cond, snip->nstr, nstr_blen, snip->encoding);
+              sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
               continue;
             }
           }
@@ -549,7 +474,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
           /* check nesting to make valid HTML */
           /* ToDo: allow <test><te>te</te><st>st</st></test> */
           if (cond->start_offset < last_tag_end) {
-            sen_bm_TunedBM(cond, snip->nstr, nstr_blen, snip->encoding);
+            sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
             continue;
           }
         }
@@ -557,7 +482,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
           /* If a keyword gets across a snippet, */
           /* it was skipped and never to be tagged. */
           cond->stopflag = 2;
-          sen_bm_TunedBM(cond, snip->nstr, nstr_blen, snip->encoding);
+          sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
         } else {
           found_cond = 1;
           if (cond->count == 0) {
@@ -576,7 +501,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
           if (++snip->tag_count >= MAX_SNIP_TAG_COUNT) {
             break;
           }
-          sen_bm_TunedBM(cond, snip->nstr, nstr_blen, snip->encoding);
+          sen_bm_TunedBM(cond, snip->nstr, nstr_blen);
         }
       }
       if (!found_cond) {
@@ -655,7 +580,7 @@ sen_snip_get_result(sen_snip *snip, const unsigned int index, char *result, unsi
       p += snip->tag_result[j].cond->opentag_len;
     }
 
-    if (snip->mapping == (sen_snip_mapping *) - 1) {
+    if (snip->mapping == (sen_snip_mapping *) -1) {
       switch (snip->string[i]) {
       case '<':
         *p++ = '&';
