@@ -29,21 +29,22 @@ struct sen_fse_header {
   char idstr[16];
   unsigned element_size;
   sen_id curr_max;
+  uint32_t reserved[10];
 };
 
 sen_fse *
-sen_fse_create(const char *path, unsigned element_size)
+sen_fse_create(const char *path, unsigned int element_size)
 {
   sen_io *io;
   int max_segments, n_elm, w_elm;
   sen_fse *fse = NULL;
   struct sen_fse_header *header;
   unsigned actual_size;
-  for (actual_size = 1; actual_size < element_size;) {
-    if ((actual_size *= 2) > SEN_FSE_SEGMENT_SIZE) {
-      SEN_LOG(sen_log_error, "element_size too large (%d)", element_size);
-    }
+  if (element_size > SEN_FSE_SEGMENT_SIZE) {
+    SEN_LOG(sen_log_error, "element_size too large (%d)", element_size);
+    return NULL;
   }
+  for (actual_size = 1; actual_size < element_size; actual_size *= 2) ;
   max_segments = ((SEN_ID_MAX + 1) / SEN_FSE_SEGMENT_SIZE) * actual_size;
   io = sen_io_create(path, sizeof(struct sen_fse_header),
                      SEN_FSE_SEGMENT_SIZE, max_segments, sen_io_auto, max_segments);
@@ -65,7 +66,7 @@ sen_fse_create(const char *path, unsigned element_size)
   return fse;
 }
 
-sen_fse * 
+sen_fse *
 sen_fse_open(const char *path)
 {
   sen_io *io;
@@ -94,7 +95,7 @@ sen_fse_open(const char *path)
 }
 
 sen_rc
-sen_fse_info(sen_fse *fse, int *element_size, sen_id *curr_max)
+sen_fse_info(sen_fse *fse, unsigned int *element_size, sen_id *curr_max)
 {
   if (!fse) { return sen_invalid_argument; }
   if (element_size) { *element_size = fse->header->element_size; }
@@ -121,28 +122,184 @@ sen_fse_remove(const char *path)
 }
 
 void *
-sen_fse_get(sen_fse *fse, sen_id key)
+sen_fse_get(sen_fse *fse, sen_id id)
 {
   void *p;
   uint16_t seg;
-  if (key > SEN_ID_MAX) { return NULL; }
-  seg = key >> fse->element_width;
+  if (id > SEN_ID_MAX) { return NULL; }
+  seg = id >> fse->element_width;
   SEN_IO_SEG_MAP(fse->io, seg, p);
   if (!p) { return NULL; }
-  if (key > fse->header->curr_max) { fse->header->curr_max = key; }
-  return (void *)(((byte *)p) + ((key & fse->element_mask) * fse->header->element_size));
+  if (id > fse->header->curr_max) { fse->header->curr_max = id; }
+  return (void *)(((byte *)p) + ((id & fse->element_mask) * fse->header->element_size));
 }
 
 void *
-sen_fse_at(sen_fse *fse, sen_id key)
+sen_fse_at(sen_fse *fse, sen_id id)
 {
   void *p;
   uint16_t seg;
-  if (key > fse->header->curr_max) { return NULL; }
-  seg = key >> fse->element_width;
+  if (id > fse->header->curr_max) { return NULL; }
+  seg = id >> fse->element_width;
   SEN_IO_SEG_MAP(fse->io, seg, p);
   if (!p) { return NULL; }
-  return (void *)(((byte *)p) + ((key & fse->element_mask) * fse->header->element_size));
+  return (void *)(((byte *)p) + ((id & fse->element_mask) * fse->header->element_size));
+}
+
+/**** variable sized elements ****/
+
+#define SEN_VSE_IDSTR "SENNA:VSE:01.00"
+#define SEN_VSE_SEGMENT_SIZE (1 << 22)
+#define SEN_VSE_MAX_SEGMENTS (1 << 16)
+
+struct sen_vse_header {
+  char idstr[16];
+  unsigned max_element_size;
+  unsigned max_segments;
+};
+
+struct _sen_vse_einfo {
+  union {
+    uint64_t ll;
+    struct {
+      uint32_t pos;
+      uint8_t size[4];
+    } s;
+    uint8_t bytes[8];
+  } u;
+};
+
+#define SEN_VSE_MAX_ELEMENT_SIZE (1 << 24)
+
+#define _ESIZE(e) ((e)->s.size[0] + ((e)->s.size[1] << 8) + ((e)->s.size[2] << 16))
+
+#if SEN_VSE_MAX_ELEMENT_SIZE > (1 << 24)
+# define ESIZE(e) (_ESIZE(e) + (((e)->s.size[3] & 0xc0) << 19))
+#else
+# define ESIZE _ESIZE
+#endif
+
+#define ELEMENT_LOG_SIZE(e) ((e)->s.size[3] & 0x1f)
+#define ELEMENT_SIZE(e) (ELEMENT_LOG_SIZE(e) ? ESIZE(e) : (((e)->s.size[3]) >> 5))
+
+sen_vse *
+sen_vse_create(const char *path, unsigned int max_element_size)
+{
+  sen_io *io;
+  int max_segments;
+  sen_vse *vse = NULL;
+  struct sen_vse_header *header;
+  unsigned actual_size;
+  if (max_element_size > SEN_VSE_SEGMENT_SIZE) {
+    SEN_LOG(sen_log_error, "max_element_size too large (%d)", max_element_size);
+    return NULL;
+  }
+  max_segments = max_element_size * 128;
+  if (max_segments > SEN_VSE_MAX_SEGMENTS) { max_segments = SEN_VSE_MAX_SEGMENTS; }
+  io = sen_io_create(path, sizeof(struct sen_vse_header),
+                     SEN_VSE_SEGMENT_SIZE, max_segments, sen_io_auto, max_segments);
+  if (!io) { return NULL; }
+  header = sen_io_header(io);
+  memcpy(header->idstr, SEN_VSE_IDSTR, 16);
+  header->max_element_size = max_element_size;
+  header->max_segments = max_segments;
+  if (!(vse = SEN_MALLOC(sizeof(sen_vse)))) {
+    sen_io_close(io);
+    return NULL;
+  }
+  vse->io = io;
+  vse->header = header;
+  return vse;
+}
+
+sen_vse *
+sen_vse_open(const char *path)
+{
+  sen_io *io;
+  sen_vse *vse = NULL;
+  struct sen_vse_header *header;
+  io = sen_io_open(path, sen_io_auto, UINT_MAX);
+  if (!io) { return NULL; }
+  header = sen_io_header(io);
+  if (memcmp(header->idstr, SEN_VSE_IDSTR, 16)) {
+    SEN_LOG(sen_log_error, "vse_idstr (%s)", header->idstr);
+    sen_io_close(io);
+    return NULL;
+  }
+  if (!(vse = SEN_MALLOC(sizeof(sen_vse)))) {
+    sen_io_close(io);
+    return NULL;
+  }
+  vse->io = io;
+  vse->header = header;
+  return vse;
+}
+
+sen_rc
+sen_vse_info(sen_vse *vse, unsigned int *max_element_size, sen_id *curr_max)
+{
+  if (!vse) { return sen_invalid_argument; }
+  return sen_success;
+}
+
+sen_rc
+sen_vse_close(sen_vse *vse)
+{
+  sen_rc rc;
+  if (!vse) { return sen_invalid_argument; }
+  rc = sen_io_close(vse->io);
+  SEN_FREE(vse);
+  return rc;
+}
+
+sen_rc
+sen_vse_remove(const char *path)
+{
+  if (!path) { return sen_invalid_argument; }
+  sen_io_remove(path);
+  return sen_success;
+}
+
+sen_rc
+sen_vse_put(sen_vse *vse, sen_id id, const void *value, int value_len, int flags)
+{
+  return sen_success;
+}
+
+int
+sen_vse_at(sen_vse *vse, sen_id id, void *valbuf, int buf_size)
+{
+  return 0;
+}
+
+const void *
+sen_vse_ref(sen_vse *vse, sen_id id, int *value_len)
+{
+  return NULL;
+}
+
+sen_rc
+sen_vse_unref(sen_vse *vse, sen_id id)
+{
+  return sen_success;
+}
+
+int
+sen_vse_size(sen_vse *vse, sen_id id)
+{
+  return 0;
+}
+
+sen_vse_einfo *
+sen_vse_alloc(sen_vse *vse, int element_size, void **value)
+{
+  return NULL;
+}
+
+sen_rc
+sen_vse_replace(sen_vse *vse, sen_id id, sen_vse_einfo *ei)
+{
+  return sen_success;
 }
 
 /**** vgram ****/
@@ -220,28 +377,28 @@ sen_store_update(sen_store *store, sen_id rid, sen_store_buf *b, sen_set *terms)
       sen_set_at(terms, t0, (void **) &u);
       vp = &(*u)->vnodes;
       for (tn = t0 + 1; tn < b->tvp; tn++) {
-	for (v = *vp; v && v->tid != *tn; v = v->cdr) ;
-	if (!v) {
-	  if (b->vpp < b->vpe) {
-	    v = b->vpp++;
-	  } else {
-	    // todo;
-	    break;
-	  }
-	  v->car = NULL;
-	  v->cdr = *vp;
-	  *vp = v;
-	  v->tid = *tn;
-	  v->vid = 0;
-	  v->freq = 0;
-	  v->len = tn - t0;
-	}
-	v->freq++;
-	if (v->vid) {
-	  vp = &v->car;
-	} else {
-	  break;
-	}
+        for (v = *vp; v && v->tid != *tn; v = v->cdr) ;
+        if (!v) {
+          if (b->vpp < b->vpe) {
+            v = b->vpp++;
+          } else {
+            // todo;
+            break;
+          }
+          v->car = NULL;
+          v->cdr = *vp;
+          *vp = v;
+          v->tid = *tn;
+          v->vid = 0;
+          v->freq = 0;
+          v->len = tn - t0;
+        }
+        v->freq++;
+        if (v->vid) {
+          vp = &v->car;
+        } else {
+          break;
+        }
       }
     }
     {
@@ -249,97 +406,96 @@ sen_store_update(sen_store *store, sen_id rid, sen_store_buf *b, sen_set *terms)
       if (!th) { return sen_memory_exhausted; }
       if (t0 == b->tvp) { SEN_LOG(sen_log_debug, "t0 == tvp"); }
       for (t0 = b->tvs; t0 < b->tvp; t0++) {
-	sen_id vid, vid0 = *t0, vid1 = 0;
-	sen_store_vnode *v, *v2 = NULL, **vp;
-	sen_set_at(terms, t0, (void **) &u);
-	vp = &(*u)->vnodes;
-	for (tn = t0 + 1; tn < b->tvp; tn++) {
-	  for (v = *vp; v; v = v->cdr) {
-	    if (!v->vid && (v->freq < 2 || v->freq * v->len < 4)) {
-	      *vp = v->cdr;
-	      v->freq = 0;
-	    }
-	    if (v->tid == *tn) { break; }
-	    vp = &v->cdr;
-	  }
-	  if (v) {
-	    if (v->freq) {
-	      v2 = v;
-	      vid1 = vid0;
-	      vid0 = v->vid;
-	    }
-	    if (v->vid) {
-	      vp = &v->car;
-	      continue;
-	    }
-	  }
-	  break;
-	}
-	if (v2) {
-	  if (!v2->vid) {
-	    vgram_key key;
-	    key.vid = vid1;
-	    key.tid = v2->tid;
-	    v2->vid = sen_sym_get(store->vgram, (char *)&key);
-	  }
-	  vid = *t0 = v2->vid * 2 + 1;
-	  memset(t0 + 1, 0, sizeof(sen_id) * v2->len);
-	  t0 += v2->len;
-	} else {
-	  vid = *t0 *= 2;
-	}
-	{
-	  int *tf;
-	  sen_set_get(th, &vid, (void **) &tf);
-	  (*tf)++;
-	}
+        sen_id vid, vid0 = *t0, vid1 = 0;
+        sen_store_vnode *v, *v2 = NULL, **vp;
+        sen_set_at(terms, t0, (void **) &u);
+        vp = &(*u)->vnodes;
+        for (tn = t0 + 1; tn < b->tvp; tn++) {
+          for (v = *vp; v; v = v->cdr) {
+            if (!v->vid && (v->freq < 2 || v->freq * v->len < 4)) {
+              *vp = v->cdr;
+              v->freq = 0;
+            }
+            if (v->tid == *tn) { break; }
+            vp = &v->cdr;
+          }
+          if (v) {
+            if (v->freq) {
+              v2 = v;
+              vid1 = vid0;
+              vid0 = v->vid;
+            }
+            if (v->vid) {
+              vp = &v->car;
+              continue;
+            }
+          }
+          break;
+        }
+        if (v2) {
+          if (!v2->vid) {
+            vgram_key key;
+            key.vid = vid1;
+            key.tid = v2->tid;
+            v2->vid = sen_sym_get(store->vgram, (char *)&key);
+          }
+          vid = *t0 = v2->vid * 2 + 1;
+          memset(t0 + 1, 0, sizeof(sen_id) * v2->len);
+          t0 += v2->len;
+        } else {
+          vid = *t0 *= 2;
+        }
+        {
+          int *tf;
+          sen_set_get(th, &vid, (void **) &tf);
+          (*tf)++;
+        }
       }
       if (!th->n_entries) { SEN_LOG(sen_log_debug, "th->n_entries == 0"); }
       {
-	int j = 0;
-	int skip = 0;
-	sen_set_eh *ehs, *ehp, *ehe;
-	sen_set_sort_optarg arg;
-	uint8_t *ps = SEN_MALLOC(b->len * 2), *pp, *pe;
-  if (!ps) {
-    sen_set_close(th);
-    return sen_memory_exhausted;
-  }
-	pp = ps;
-	pe = ps + b->len * 2;
-	arg.mode = sen_sort_descending;
-	arg.compar = NULL;
-	arg.compar_arg = (void *)(intptr_t)sizeof(sen_id);
-	arg.compar_arg0 = NULL;
-	ehs = sen_set_sort(th, 0, &arg);
-	if (!ehs) {
-	  SEN_FREE(ps);
-    sen_set_close(th);
-    return sen_memory_exhausted;
-	
-  }
-	SEN_B_ENC(th->n_entries, pp);
-	for (ehp = ehs, ehe = ehs + th->n_entries; ehp < ehe; ehp++, j++) {
-	  int *id = (int *)SEN_SET_INTVAL(*ehp);
-	  SEN_B_ENC(*SEN_SET_INTKEY(*ehp), pp);
-	  *id = j;
-	}
-	for (t0 = b->tvs; t0 < b->tvp; t0++) {
-	  if (*t0) {
-	    int *id;
-	    if (!sen_set_at(th, t0, (void **) &id)) {
-	      SEN_LOG(sen_log_error, "lookup error (%d)", *t0);
-	    }
-	    SEN_B_ENC(*id, pp);
-	  } else {
-	    skip++;
-	  }
-	}
-	len_sum += b->len;
-	img_sum += pp - ps;
-	skip_sum += skip;
-	SEN_FREE(ehs);
-	SEN_FREE(ps);
+        int j = 0;
+        int skip = 0;
+        sen_set_eh *ehs, *ehp, *ehe;
+        sen_set_sort_optarg arg;
+        uint8_t *ps = SEN_MALLOC(b->len * 2), *pp, *pe;
+        if (!ps) {
+          sen_set_close(th);
+          return sen_memory_exhausted;
+        }
+        pp = ps;
+        pe = ps + b->len * 2;
+        arg.mode = sen_sort_descending;
+        arg.compar = NULL;
+        arg.compar_arg = (void *)(intptr_t)sizeof(sen_id);
+        arg.compar_arg0 = NULL;
+        ehs = sen_set_sort(th, 0, &arg);
+        if (!ehs) {
+          SEN_FREE(ps);
+          sen_set_close(th);
+          return sen_memory_exhausted;
+        }
+        SEN_B_ENC(th->n_entries, pp);
+        for (ehp = ehs, ehe = ehs + th->n_entries; ehp < ehe; ehp++, j++) {
+          int *id = (int *)SEN_SET_INTVAL(*ehp);
+          SEN_B_ENC(*SEN_SET_INTKEY(*ehp), pp);
+          *id = j;
+        }
+        for (t0 = b->tvs; t0 < b->tvp; t0++) {
+          if (*t0) {
+            int *id;
+            if (!sen_set_at(th, t0, (void **) &id)) {
+              SEN_LOG(sen_log_error, "lookup error (%d)", *t0);
+            }
+            SEN_B_ENC(*id, pp);
+          } else {
+            skip++;
+          }
+        }
+        len_sum += b->len;
+        img_sum += pp - ps;
+        skip_sum += skip;
+        SEN_FREE(ehs);
+        SEN_FREE(ps);
       }
       sen_set_close(th);
     }
